clear all
close all
clc

%%
%sampling frequency
Fs = 1024;
%sampline time
dT = 1/Fs;
% Signal length
L = 1024;
% time axis
t = (0:L-1)*dT;
% frequency axis
f = (-Fs/2 : Fs/L : Fs/2-Fs/L);



%% Shift-Invariance

% rect signals
rec1 = zeros(size(t));
rec1(t>0.1 & t < 0.4)=1;

rec2 = zeros(size(t));
rec2(t>0.5 & t < 0.8)=1;

figure
subplot(2,2,1)
plot(t,real(rec1));
xlim([min(t),max(t)])
xlabel('time [s]')
ylim([-0.05 1.05])
subplot(2,2,2)
plot(f,abs(fftshift(fft(rec1))));
xlim([-50,50])
xlabel('|frequency| [Hz]')
subplot(2,2,3)
plot(t,real(rec2));
xlim([min(t),max(t)])
xlabel('time [s]')
ylim([-0.05 1.05])
subplot(2,2,4)
plot(f,abs(fftshift(fft(rec2))));
xlim([-50,50])
xlabel('|frequency| [Hz]')



%% Rotation stays rotation (2D)

[X,Y] = meshgrid(t,t);
img = zeros(size(X));
img(X>0.4 & X < 0.8 & Y > 0.675 & Y < 0.7)=1;

figure
subplot(2,2,1)
imagesc(imrotate(img,-45,'nearest','crop'))
axis image
subplot(2,2,2)
imagesc(log(abs(fftshift(fft2(imrotate(img,-45,'nearest','crop'))))+1));
axis image
subplot(2,2,3)
imagesc(imrotate(img,45,'nearest','crop'))
axis image
subplot(2,2,4)
imagesc(log(abs(fftshift(fft2(imrotate(img,45,'nearest','crop'),2048,2048)))+1));
axis image
colormap gray




%% Real signals and symmetry F(gamma)=F*(-gamma)

realSignal = 2*t.^3+4*t.^2-1.3;

figure
subplot(2,1,1)
stem(f,real(fftshift(fft(realSignal))))
xlim([-50,50])
xlabel('frequency [Hz]')
title('Real part of FFT is symmetric')
subplot(2,1,2)
stem(f,imag(fftshift(fft(realSignal))))
xlim([-50,50])
xlabel('frequency [Hz]')
title('Imaginary part of FFT is point-symmetric')



%% Convolution theorem f(t)g(t) = F(f)*G(f) or vice versa

% gaussian convolution
filterkernel = fspecial('gaussian',[1,length(t)],5);
% a simple signal
sig = cos(2*pi*5*t) + 0.2*randn(1,length(t));

figure
subplot(3,2,1)
plot(t,sig)
xlim([min(t),max(t)])
xlabel('time [s]')
title('f(t)')
subplot(3,2,3)
plot(t,filterkernel)
xlim([min(t),max(t)])
xlabel('time [s]')
title('g(t)')
subplot(3,2,5)
plot(t,imfilter(sig, filterkernel));
xlim([min(t),max(t)])
xlabel('time [s]')
title('conv(f(t),g(t))')

subplot(3,2,2)
plot(f,abs(fftshift(fft(sig))))
xlim([-75,75])
xlabel('|frequency| [Hz]')
title('F(f)')
subplot(3,2,4)
plot(f,abs(fftshift(fft(filterkernel))));
xlim([-75,75])
xlabel('|frequency| [Hz]')
title('G(f)')
subplot(3,2,6)
res=ifft(fft(sig,2*L).*fft(filterkernel,2*L));
plot(t,res(end/2-L/2+1:end/2+L/2));
xlim([min(t),max(t)])
xlabel('time [s]')
title('ifft(F(f).*G(f))')



%% frequency vs. time resolution

sigmas = 2.^linspace(0.000000001, 10,100)-1;
figure
for i=1:length(sigmas)
    gauss = fspecial('gaussian', [1,L], sigmas(i));
    subplot(2,1,1)
    plot(t,mat2gray(gauss));
    xlim([min(t),max(t)])
    xlabel('time [s]')
    ylim([-0.05 1.05])
    subplot(2,1,2)
    plot(f,abs(fftshift(fft(gauss))))
    xlabel('|frequency| [Hz]')
    ylim([-0.05 1.05])
    pause(0.1)
end



%% Filtering in frequency domain

% Lets define an "easy" signal
sig = cos(2*pi*10*t) + cos(2*pi*20*t) + sin(2*pi*30*t);

% fourier transform of signal
SIG = fftshift(fft(sig));

% lets filter the highest frequencies
SIGfilt=SIG;
SIGfilt(abs(f)>15)=0;
% inverse fourier transform
sigfilt=ifft(ifftshift(SIGfilt));

% plot things
figure
subplot(2,2,1)
plot(t,real(sig));
xlim([min(t),max(t)])
xlabel('time [s]')
title('cos(2\pi10t) + cos(2\pi20t) + sin(2\pi30t)')
ylim([-3 3])
subplot(2,2,2)
stem(f,abs(SIG));
xlim([-50,50])
xlabel('|frequency| [Hz]')
subplot(2,2,3)
plot(t,real(sigfilt));
xlim([min(t),max(t)])
ylim([-1.1 1.1])
xlabel('time [s]')
title('\approx cos(2\pi10t)')
subplot(2,2,4)
stem(f,abs(SIGfilt));
xlim([-50,50])
xlabel('|frequency| [Hz]')














